import {Component, Input, OnChanges, OnDestroy, OnInit, SimpleChanges} from '@angular/core';
import {Product, ProductType} from "../../../model/product";
import {FormControl, FormGroup} from "@angular/forms";
import {Observable, Subject, Subscription} from "rxjs";
import {ProductsService} from "../../../services/products.service";
import {JobsService} from "../../../services/jobs.service";
import {faChevronCircleRight, faCubes, faEdit, faFileCode, faList} from "@fortawesome/free-solid-svg-icons";
import {AlertService} from "../../../services/alert.service";
import {takeUntil} from "rxjs/operators";
import {Action} from "../../../model/action";
import {ActionsService} from "../../../services/actions.service";

@Component({
  selector: 'app-product-details',
  templateUrl: './product-details.component.html',
  styleUrls: ['./product-details.component.scss']
})
export class ProductDetailsComponent implements OnInit, OnChanges, OnDestroy {

  private ngUnsubscribe = new Subject();
  @Input() product: Product;

  type: ProductType;

  typeConfig: Observable<object>;
  productConfig: Observable<object>;
  mergedConfig: Observable<object>;

  showProductForm: boolean = false;
  jobFormGroup: FormGroup;

  public queues: Array<any> = [];
  public queues$: Subscription;

  public faEdit = faEdit;
  public faList = faList;
  public faCubes = faCubes;
  public faFileCode = faFileCode;
  public faChevronCircleRight = faChevronCircleRight;

  private lastAction: Action;

  constructor(
    private productService: ProductsService,
    private jobsService: JobsService,
    private alertService: AlertService,
    private actionsService: ActionsService
  ) {
    this.jobFormGroup = new FormGroup({
      queue: new FormControl(),
      version: new FormControl()
    });
  }

  ngOnInit() {
    this.actionsService.actionsTaken$.pipe(takeUntil(this.ngUnsubscribe)).subscribe((actions: Array<Action>) => {
      actions.forEach((a: Action) => {
        if (a.type === 'jobSubmittedFromProduct' && a.id === this.product.id) {
          this.lastAction = a;
        }
      });
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.hasOwnProperty('product')) {
      if (changes.product.currentValue.preRequisites.length > 0 &&
        changes.product.currentValue.preRequisites[0].requiredProduct.versions.length > 0) {
        let defaultVersion = changes.product.currentValue.preRequisites[0].requiredProduct.versions[0].versionNumber;
        for (let version of changes.product.currentValue.preRequisites[0].requiredProduct.versions) {
          if (version.status === 'QA_ACCEPTED' && defaultVersion < version.versionNumber) {
            defaultVersion = version.versionNumber;
          }
        }
        this.jobFormGroup.get('version').setValue(defaultVersion);
      }
      // product.preRequisites[0].requiredProduct.versions.reverse()
      this.productConfig = this.productService.getProductConfig(changes.product.currentValue.id, 'configurations');
      this.mergedConfig = this.productService.getProductConfig(changes.product.currentValue.id, 'merged');
      const productType = Product.getTypeFromName(changes.product.currentValue.typeName);
      if (productType) {
        this.type = productType;
        this.queues$ = this.productService.getPageInfo(productType.id).pipe(takeUntil(this.ngUnsubscribe)).subscribe((queues: Array<string>) => {
          this.queues = queues;
          this.jobFormGroup.get('queue').setValue(queues[0]);
        });
        this.typeConfig = this.productService.getProductTypeConfig(productType.id);
      }
    }
  }

  canBeSubmitted(): boolean {
    return this.product.status === 'READY' && (!this.lastAction || this.lastAction.timeSince > (1000 * 60 * 5));
  }

  createJob() {
    this.alertService.info('Creating Job...');
    let queue = this.jobFormGroup.get('queue').value;
    let version = parseInt(this.jobFormGroup.get('version').value);
    if (!version || isNaN(version)) {
      version = null;
    }
    this.actionsService.addAction('jobSubmittedFromProduct', this.product.id);
    this.jobsService.createJob(this.product.id, queue, version).subscribe(response => {
      this.alertService.success('Job created');
    }, (error) => {
      this.actionsService.removeActionByTypeAndId('jobSubmittedFromProduct', this.product.id);
      this.alertService.error('Job creation failed');
    });
  }

  ngOnDestroy(): void {
    this.ngUnsubscribe.next();
    this.ngUnsubscribe.complete();
  }

}
